<?php
// $Id: ShindigIntegratorDbFetcher.php,v 1.1.2.4 2009/01/06 07:26:59 impetus Exp $
/**
 * @file
 * DB interaction layer for shindig services
 *
 * @see http://incubator.apache.org/shindig/
 * This module contains core shindig server
 */

include(Config::get('settings_php'));
include_once(Config::get('include_path')."/database.inc");
include_once(Config::get('include_path')."/bootstrap.inc");
include_once(Config::get('include_path')."/common.inc");

$url_array = parse_url($db_url);
$GLOBALS['url_array'] = $url_array;
include_once(Config::get('include_path')."/database.".$url_array['scheme'].".inc");

global $active_db;
$active_db = db_connect($db_url);

class ShindigIntegratorDbFetcher {
	private $db;
	private $url_prefix;
	
	// Singleton
	private static $fetcher;

	private function __construct()
	{
		global $url_array;
		$this->url_prefix = "http://" . $_SERVER['HTTP_HOST'] . $url_array['path']."/";
	}

	private function __clone()
	{
		// private, don't allow cloning of a singleton
	}

	static function get()
	{
		// This object is a singleton
		if (! isset(ShindigIntegratorDbFetcher::$fetcher)) {
			ShindigIntegratorDbFetcher::$fetcher = new ShindigIntegratorDbFetcher();
		}
		return ShindigIntegratorDbFetcher::$fetcher;
	}

	/**
	* To get Friends IDs for the given UserID
	*
	* @param
	*   $userId for who friends are being retrived
	* @return
	*   $ret ids of all the friends of the user
	*/
	public function getFriendIds($userId)
	{
		$ret = array();
		$res = db_query("SELECT requester_id, requestee_id FROM {user_relationships} WHERE requester_id = %d OR requestee_id = %d and rtid=(SELECT rtid FROM {user_relationship_types} WHERE name='friend') AND approved=1", $userId, $userId);
		while ($row = db_fetch_array($res)) {
			if($row['requester_id']==$userId)
			{
				if(!in_array($row['requestee_id'],$ret))
    			    $ret[] = $row['requestee_id'] ;
			}
			else
			{
    			if(!in_array($row['requester_id'],$ret))
                    $ret[] = $row['requester_id'] ;
			}
		}
		return $ret;
	}    
    
	/**
	* Get User(s) 
	* Return array of Person Model Objects
	* 
	* @param
	*   $ids of the userIds
	* @param
	*   $profileDetails profile fields to fetched for the users
	* @param
	*   $options other query options to get Users
	* @return
	*   $ret array of person objects 
	*/
	public function getPeople($ids, $profileDetails, $options)
	{
		$ret = array();
		foreach($ids as $id)
		{
			$user =array();
			$res = db_query("SELECT * FROM {profile_values} INNER JOIN profile_fields ON profile_values.fid = profile_fields.fid WHERE uid =%d", $id);		
			if ($res) {
				while ($row = db_fetch_array($res)) 
				{
					$user[$row['name']] = $row['value'];
					$user['uid'] = $row['uid'];
				}			
			}	
			$user_id = $user['uid'];
			$name = new Name($user['profile_fname'] . ' ' . $user['profile_lname']);
			$name->setGivenName($user['profile_fname']);
			$name->setFamilyName($user['profile_lname']);
			$person = new Person($user['uid'], $name);
			$person->setProfileUrl($this->url_prefix . '?q=user/' . $user['uid']);
            $person->setDisplayName($user['profile_fname'] . ' ' . $user['profile_lname']);
			$res = db_query("SELECT picture FROM {users} WHERE uid = %d",$id);	
			$row =  db_fetch_array($res);
			$person->setThumbnailUrl(! empty($row['picture']) ? $this->url_prefix.$row['picture'] : '');
						if ($user['profile_gender'] == 'Female') {
						    $person->setGender('FEMALE');
					    } else {
						    $person->setGender('MALE');
					    }
	
			
				$ret[$user_id] = $person;
			
		}
		return $ret;
	}
	
	/**
	* Get User Id(s) who have specific application
	* Return array of Person Model Objects
	* 
	* @param
	*   $appId application id
	* @return
	*   $peopleWithApp array of person ids
	*/
	public function getPeopleWithApp($appId) {
		$peopleWithApp = array();
		$res = db_query("SELECT user_id FROM user_applications WHERE application_id=%d", $appId);
		while ($row = db_fetch_array($res)) {			
				$peopleWithApp[] = $row['user_id'];
		}
		return $peopleWithApp;
	}
	
		/**
	* To save PersonAppData into database
	*
	* @param
	*   $userId for who data is to be stored
    * @param
	*   $key name of the Appdata 
	* @param
	*   $value value of the Appdata 
	* @param
	*   $appId application to which Appdata belongs to
	*/
	public function setAppData($userId, $key, $value, $appId)
	{
		if (empty($value)) {
			if (! db_query("DELETE FROM {application_settings} WHERE application_id = %d AND user_id = %d AND name = '%s'", $appId, $userId, $key)) {
				return false;
			}
		} else {
			if (! db_query("INSERT INTO {application_settings} (application_id, user_id, module_id, name, value) VALUES (%d, %d, 0, '%s', '%s') ON DUPLICATE KEY UPDATE value = '%s'", $appId, $userId, $key, $value, $value)) {
				return false;
			}
		}
		return true;
	}
    
	/**
	* To get PersonAppData
	*
	* @param
	*   $ids userIds 
    * @param
	*   $key name of the Appdata 
	* @param
	*   $appId application to which Appdata belongs to
	* @return
	*   $data Appdata
	*/	
	public function getAppData($ids, $keys, $app_id) {
    $data = array();
    $placeholders_ids = array_fill(0, count($ids), "%d");
    if (in_array("@all", $keys)) {
      $res = db_query("SELECT user_id, name, value FROM application_settings WHERE application_id = %d AND user_id IN (" . implode(',', $placeholders_ids) . ")", $app_id, $ids);
    } 
	else {
      $placeholders_keys = array_fill(0, count($keys), "%s");
      $values = array();
      $values[] = $app_id;
      $values = array_merge($values, $ids);
      $values = array_merge($values, $keys);
      $res = db_query("SELECT user_id, name, value FROM application_settings WHERE application_id = %d 
                        AND user_id IN (" . implode(',', $placeholders_ids) . ") 
						AND name IN ('". implode(',', $placeholders_keys )."')", $values);
    }
    while ($app_data = db_fetch_array($res)) {
      $user_id = $app_data['user_id'];
      if (! isset($user_id)) {
        $data[$user_id] = array();
      }
      $key = $app_data['name'];
      $value = $app_data['value'];
      $data[$user_id][$key] = $value;
    }
    return $data;
  }
    
	/**
	* Delete PersonAppData
	*
	* @param
	*   $userId for who data is to be deleted
    * @param
	*   $key name of the Appdata 
	* @param
	*   $appId application to which Appdata belongs to
	*/
	public function deleteAppData($userId, $key, $appId)
	{
		if(!db_query("DELETE FROM {application_settings} WHERE application_id = %d AND user_id = %d AND name = '%s'", $appId, $userId, $key)) {
			return false;
		}
		return true;

	}
	
	/**
	* To Store User's Activity in Db
	*
	* @param
	*   $userId for who Activity is to be created
    * @param
	*   $activity activity to be created
	* @param
	*   $appId application to which Activity belongs to
	*/
	public function createActivity($user_id, $activity, $app_id = '0') {
    $title = isset($activity['title']) ? trim($activity['title']) : '';
    $body = isset($activity['body']) ? trim($activity['body']) : '';
    $time = time();
    $activityId = 0;
    $params = array("title" => $title, "body" => $body, "app_id" => $app_id, "user_id" => $user_id);
    $request = xmlrpc_encode_request("shindig_integrator.save_activities", $params);
    $context = stream_context_create(array('http' => array(
                                           'method' => "POST",
                                           'header' => "Content-Type: text/xml",
                                           'content' => $request)
                                          )
                                    );
    $file = file_get_contents("http://". $_SERVER['HTTP_HOST'] . Config::get('base_path')."/xmlrpc.php", false, $context);
    $response = xmlrpc_decode($file);
    if (! xmlrpc_is_fault($response)) {		   
      $activityId = $response;
    }
    if($activityId) {
      $mediaItems = isset($activity['mediaItems']) ? $activity['mediaItems'] :  array();
      if (count($mediaItems)) {
        foreach ($mediaItems as $mediaItem) {
          $type = isset($mediaItem['type']) ? $mediaItem['type'] : '';
          $mimeType = isset($mediaItem['mimeType']) ? $mediaItem['mimeType'] : '';
          $url = isset($mediaItem['url']) ? $mediaItem['url'] : '';
          $type = trim($type);
          $mimeType = trim($mimeType);
          $url = trim($url);
          db_query("INSERT INTO activity_media_items (id, activity_id, mime_type, media_type, url)
                    VALUES (0, %d, '%s', '%s', '%s')", $activityId, $mimeType, $type, $url);			
        }
      }
      return true;
    }		
    return false;		
  }

    
	/**
	* To get Activities of the User(s)
	*
	* @param
	*   $ids userids - for all activites needs to be fetched
	* @param
	*   $app_id application to which Activity belongs to
	* @param
	*   $sortBy to sort result sets by
	* @param
	*   $filterBy to filter activity results
	* @param
	*   $startIndex start index for pagination of results
    * @param
	*   $count number of results on the page
    * @param
	*   $fields activity fields in the results
	* @return
	*   $activities array of activity objects
	*/
	public function getActivities($userId, $ids, $appId, $sortBy, $filterBy, $startIndex, $count, $fields) {
    $activities = array();
    $params = array("user_id" => $userId, "app_id" => $appId, "ids" => $ids);
    $request = xmlrpc_encode_request("shindig_integrator.get_activities", $params);
    $context = stream_context_create(array('http' => array(
                                           'method' => "POST",
                                           'header' => "Content-Type: text/xml",
                                           'content' => $request)
                                          )
                                    );
    $file = file_get_contents("http://". $_SERVER['HTTP_HOST'] . Config::get('base_path')."/xmlrpc.php", false, $context);
    $response = xmlrpc_decode($file);
    foreach($response as $activityData) {
      $activity = new Activity($activityData['activity_id'], $activityData['user_id']);
      $activity->setStreamTitle('activities');
      $activity->setTitle($activityData['title']);
      $activity->setBody($activityData['body']);
      $activity->setPostedTime($activityData['created']);
      $activity->setMediaItems($this->getMediaItems($activityData['activity_id']));
      $activities[] = $activity;
    }
    return $activities;
  }
    
	/**
	* To get Mediaitems of an Activity
	*
	* @param
	*   $activityId for which media items is to fetched
	* @return
	*   $media atcivity media item object  
	*/  
	private function getMediaItems($activityId)
	{
		$media = array();
		$res = db_query("SELECT mime_type, media_type, url FROM {activity_media_items} WHERE activity_id = %d", $activityId);
		while ($row = db_fetch_array($res)) {
			$media[] = new MediaItem($row['mime_type'], $row['media_type'], $row['url']);
		}
		return $media;
	}

	/**
	* Get the set of user id's from a user or collection of users, and group
	*/
	public function getIdSet($user, GroupId $group, SecurityToken $token)
	{
		$ids = array();
		if ($user instanceof UserId) {
			$userId = $user->getUserId($token);
			if ($group == null) {
				return array($userId);
			}
			switch ($group->getType()) {
				case 'all':
				case 'friends':
				case 'groupId':
					$friendIds = ShindigIntegratorDbFetcher::get()->getFriendIds($userId);
					if (is_array($friendIds) && count($friendIds)) {
						$ids = $friendIds;
					}
					break;
				case 'self':
					$ids[] = $userId;
					break;
			}
		} elseif (is_array($user)) {
			$ids = array();
			foreach ($user as $id) {
				$ids = array_merge($ids, $this->getIdSet($id, $group, $token));
			}
		}
		return $ids;
	}


}
?>

